@@ -44,7 +44,6 @@ LOCAL_MEDIA = []
 
 # GeoIP/managed URLs
 FEDORA_BASEURL = "http://download.fedoraproject.org/pub/fedora/linux/releases/%s/Fedora/%s/os/"
-FEDORA_RAWHIDE_BASEURL = "http://download.fedoraproject.org/pub/fedora/linux/development/%s/os"
 OPENSUSE_BASEURL = "http://download.opensuse.org/distribution/%s/repo/oss/"
 OLD_OPENSUSE_BASEURL = "http://ftp5.gwdg.de/pub/opensuse/discontinued/distribution/%s/repo/oss"
 
@@ -59,7 +58,7 @@ MANDRIVA_BASEURL = "http://ftp.uwsg.indiana.edu/linux/mandrake/official/%s/%s/"
 SCIENTIFIC_BASEURL = "http://ftp.scientificlinux.org/linux/scientific/%s/%s/"
 
 # Doesn't appear to be a simple boot iso in newer suse trees
-NOBOOTISO_FILTER = ".*opensuse11.*|.*opensuse10.3.*|.*opensuse10.0.*"
+NOBOOTISO_FILTER = ".*opensuse12.*|.*opensuse11.*|.*opensuse10.3.*|.*opensuse10.0.*"
 
 # Opensuse < 10.3 (and some sles) require crazy rpm hacking to get a bootable
 # kernel. We expect failure in this case since our test harness doesn't
@@ -88,19 +87,14 @@ def distroClass(distname):
 urls = {
 
     # Fedora Distros
-    "fedora13" : {
-        'x86_64': FEDORA_BASEURL % ("13", "x86_64"),
-        'distro': ("linux", "fedora13")
-    },
-    "fedora14" : {
-        'x86_64': FEDORA_BASEURL % ("14", "x86_64"),
-        'distro': ("linux", "fedora14")
-    },
-    "fedora-rawhide" : {
-        'i386'  : FEDORA_RAWHIDE_BASEURL % ("i386"),
-        'x86_64': FEDORA_RAWHIDE_BASEURL % ("x86_64"),
+    "fedora15" : {
+        'x86_64': FEDORA_BASEURL % ("15", "x86_64"),
         'distro': ("linux", "fedora15")
     },
+    "fedora16" : {
+        'x86_64': FEDORA_BASEURL % ("16", "x86_64"),
+        'distro': ("linux", "fedora16")
+    },
 
     # SUSE Distros
     "opensuse10.0" : {
@@ -117,6 +111,10 @@ urls = {
         'i386'  : OPENSUSE_BASEURL % ("11.4"),
         'x86_64': OPENSUSE_BASEURL % ("11.4")
     },
+    "opensuse12.1" : {
+        'i386'  : OPENSUSE_BASEURL % ("12.1"),
+        'x86_64': OPENSUSE_BASEURL % ("12.1")
+    },
 
     # Debian Distros
     "debian-lenny-64" : {
@@ -133,6 +131,10 @@ urls = {
         'x86_64': DEBIAN_BASEURL % ("wheezy", "amd64"),
         'distro': ("linux", None)
     },
+    "debian-sid" : {
+        'x86_64': DEBIAN_BASEURL % ("sid", "amd64"),
+        'distro': ("linux", None)
+    },
     "debian-daily" : {
         'i386' : "http://people.debian.org/~joeyh/d-i/",
         'distro': ("linux", None)
@@ -189,11 +191,6 @@ urls = {
         'x86_64': UBUNTU_BASEURL % ("hardy", "amd64"),
         'distro': ("linux", None)
     },
-    "ubuntu-lucid" : {
-        'i386': UBUNTU_BASEURL % ("lucid", "i386"),
-        'x86_64': UBUNTU_BASEURL % ("lucid", "amd64"),
-        'distro': ("linux", None)
-    },
     "ubuntu-maverick" : {
         'i386': UBUNTU_BASEURL % ("maverick", "i386"),
         'x86_64': UBUNTU_BASEURL % ("maverick", "amd64"),
@@ -204,17 +201,27 @@ urls = {
         'x86_64': UBUNTU_BASEURL % ("natty", "amd64"),
         'distro': ("linux", None)
     },
+    "ubuntu-oneiric" : {
+        'i386': UBUNTU_BASEURL % ("oneiric", "i386"),
+        'x86_64': UBUNTU_BASEURL % ("oneiric", "amd64"),
+        'distro': ("linux", None)
+    },
+    "ubuntu-precise" : {
+        'i386': UBUNTU_BASEURL % ("precise", "i386"),
+        'x86_64': UBUNTU_BASEURL % ("precise", "amd64"),
+        'distro': ("linux", None)
+    },
 
     # Mandriva
     "mandriva-2009.1" : {
         "noxen": True,
-        'i386': MANDRIVA_BASEURL % ("2009.1", "i586"),
+        'i586': MANDRIVA_BASEURL % ("2009.1", "i586"),
         'x86_64': MANDRIVA_BASEURL % ("2009.1", "x86_64"),
         'distro': ("linux", None)
     },
     "mandriva-2010.2" : {
         "noxen": True,
-        'i386': MANDRIVA_BASEURL % ("2010.2", "i586"),
+        'i586': MANDRIVA_BASEURL % ("2010.2", "i586"),
         'x86_64': MANDRIVA_BASEURL % ("2010.2", "x86_64"),
         'distro': ("linux", None)
     },
@@ -242,22 +249,22 @@ class TestURLFetch(unittest.TestCase):
 
             # Make sure we detect _a_ distro
             hvmstore = self._getStore(fetcher, mediapath, "hvm", arch)
-            logging.debug("Local distro detected as: %s" % hvmstore)
+            logging.debug("Local distro detected as: %s", hvmstore)
         finally:
             fetcher.cleanupLocation()
 
 
     def _fetchFromURLDict(self, distname, url, arch, distro_info, check_xen):
-        logging.debug("\nDistro='%s' arch='%s' url=%s" % \
-                      (distname, arch, url))
+        logging.debug("\nDistro='%s' arch='%s' url=%s",
+                      distname, arch, url)
 
         fetcher = OSDistro._fetcherForURI(url, "/tmp")
         try:
             fetcher.prepareLocation()
         except Exception, e:
             # Don't raise an error here: the site might be down atm
-            logging.error("%s-%s: Couldn't access url %s: %s. Skipping." % \
-                          (distname, arch, fetcher.location, str(e)))
+            logging.error("%s-%s: Couldn't access url %s: %s. Skipping.",
+                          distname, arch, fetcher.location, str(e))
             fetcher.cleanupLocation()
             return
 
@@ -303,8 +310,8 @@ class TestURLFetch(unittest.TestCase):
         exp_store = distroClass(distname)
         for s in [hvmstore, xenstore]:
             if s and not isinstance(s, exp_store):
-                logging.error("(%s): expected store %s, was %s" % \
-                              (distname, exp_store, s))
+                logging.error("(%s): expected store %s, was %s",
+                              distname, exp_store, s)
                 self.fail()
 
         # Make sure the stores are reporting correct distro name/variant
@@ -318,7 +325,7 @@ class TestURLFetch(unittest.TestCase):
             if not isinstance(meter, urlgrabber.progress.BaseMeter):
                 raise ValueError("passed meter is '%s' not an"
                                  " actual meter." % meter)
-            logging.debug("Fake acquiring %s" % filename)
+            logging.debug("Fake acquiring %s", filename)
             return fetcher.hasFile(filename)
 
         # Replace acquireFile with hasFile, so we don't actually have to fetch
@@ -328,36 +335,36 @@ class TestURLFetch(unittest.TestCase):
         # Fetch boot iso
         try:
             if re.match(r"%s" % NOBOOTISO_FILTER, distname):
-                logging.debug("Known lack of boot.iso in %s tree. Skipping." \
-                              % distname)
+                logging.debug("Known lack of boot.iso in %s tree. Skipping.",
+                              distname)
             else:
                 boot = hvmstore.acquireBootDisk(testguest, fetcher, self.meter)
-                logging.debug("acquireBootDisk: %s" % str(boot))
+                logging.debug("acquireBootDisk: %s", str(boot))
 
                 if boot != True:
                     raise RuntimeError("Didn't fetch any boot iso.")
         except Exception, e:
-            logging.exception("%s-%s: bootdisk fetching: %s" %
-                              (distname, arch, str(e)))
+            logging.exception("%s-%s: bootdisk fetching: %s",
+                              distname, arch, str(e))
             self.fail()
 
         # Fetch regular kernel
         try:
             kern = hvmstore.acquireKernel(testguest, fetcher, self.meter)
-            logging.debug("acquireKernel (hvm): %s" % str(kern))
+            logging.debug("acquireKernel (hvm): %s", str(kern))
 
             if kern[0] is not True or kern[1] is not True:
                 raise RuntimeError("Didn't fetch any hvm kernel.")
         except Exception, e:
-            logging.exception("%s-%s: hvm kernel fetching: %s" %
-                              (distname, arch, str(e)))
+            logging.exception("%s-%s: hvm kernel fetching: %s",
+                              distname, arch, str(e))
             self.fail()
 
         # Fetch xen kernel
         try:
             if xenstore and check_xen:
                 kern = xenstore.acquireKernel(testguest, fetcher, self.meter)
-                logging.debug("acquireKernel (xen): %s" % str(kern))
+                logging.debug("acquireKernel (xen): %s", str(kern))
 
                 if kern[0] is not True or kern[1] is not True:
                     raise RuntimeError("Didn't fetch any xen kernel.")
@@ -365,10 +372,10 @@ class TestURLFetch(unittest.TestCase):
                 logging.debug("acquireKernel (xen): Hardcoded skipping.")
         except Exception, e:
             if re.match(r"%s" % EXPECT_XEN_FAIL, distname):
-                logging.debug("%s: anticipated xen failure." % distname)
+                logging.debug("%s: anticipated xen failure.", distname)
             else:
-                logging.exception("%s-%s: xen kernel fetching: %s" %
-                                  (distname, arch, str(e)))
+                logging.exception("%s-%s: xen kernel fetching: %s",
+                                  distname, arch, str(e))
                 self.fail()
 
     def _getStore(self, fetcher, url, _type, arch):
@@ -379,7 +386,7 @@ class TestURLFetch(unittest.TestCase):
                                                 arch=arch, typ=_type)
             except Exception, e:
                 if str(e).count("502"):
-                    logging.debug("Caught proxy error: %s" % str(e))
+                    logging.debug("Caught proxy error: %s", str(e))
                     time.sleep(.5)
                     continue
                 raise
@@ -397,7 +404,7 @@ class TestURLFetch(unittest.TestCase):
         for label in keys:
             distro_info = None
             if MATCH_FILTER and not re.match(r"%s" % MATCH_FILTER, label):
-                logging.debug("Excluding '%s' from exclude filter." % label)
+                logging.debug("Excluding '%s' from exclude filter.", label)
                 continue
 
             check_xen = not bool(urls[label].get("noxen"))
@@ -430,7 +437,7 @@ class TestURLFetch(unittest.TestCase):
                 try:
                     self._fetchLocalMedia(p)
                 except Exception, e:
-                    logging.exception("Local path '%s' failed: %s" % (p, e))
+                    logging.exception("Local path '%s' failed: %s", p, e)
                     print "Local path FAILED."
                     assertions += 1
 
