@@ -53,7 +53,7 @@ class TestXMLConfig(unittest.TestCase):
 
     def setUp(self):
         utils.reset_conn()
-        logging.debug("Running %s" % self.id())
+        logging.debug("Running %s", self.id())
 
     def tearDown(self):
         if os.path.exists(utils.scratch):
@@ -666,11 +666,11 @@ class TestXMLConfig(unittest.TestCase):
         dev6 = VirtualCharDevice.get_dev_instance(g.conn,
                                                   VirtualCharDevice.DEV_CONSOLE,
                                                   VirtualCharDevice.CHAR_PTY)
-        dev6.target_type = dev5.CHAR_CONSOLE_TARGET_VIRTIO
 
         dev7 = VirtualCharDevice.get_dev_instance(g.conn,
                                                   VirtualCharDevice.DEV_CONSOLE,
                                                   VirtualCharDevice.CHAR_PTY)
+        dev7.target_type = dev5.CHAR_CONSOLE_TARGET_VIRTIO
 
         dev8 = VirtualCharDevice.get_dev_instance(g.conn,
                                                   VirtualCharDevice.DEV_CHANNEL,
@@ -694,6 +694,7 @@ class TestXMLConfig(unittest.TestCase):
         g = utils.get_basic_fullyvirt_guest(installer=i)
 
         g.description = "foooo barrrr somedesc"
+        g.hugepage = True
 
         # Hostdevs
         dev1 = VirtualHostDeviceUSB(g.conn)
@@ -847,6 +848,13 @@ class TestXMLConfig(unittest.TestCase):
         cpu = virtinst.CPU(g.conn)
         self.assertEquals(cpu.vcpus_from_topology(), 1)
 
+    def testUsb2(self):
+        i = utils.make_pxe_installer()
+        g = utils.get_basic_fullyvirt_guest(installer=i)
+
+        g.add_usb_ich9_controllers()
+
+        self._compare(g, "boot-usb2", False)
 
     #
     # Full Install tests: try to mimic virt-install as much as possible
@@ -866,7 +874,15 @@ class TestXMLConfig(unittest.TestCase):
         g.add_device(VirtualVideoDevice(g.conn))
         g.os_autodetect = True
 
-        self._testInstall(g, "rhel6-kvm-stage1", "rhel6-kvm-stage2")
+        # Do this ugly hack to make sure the test doesn't try and use vol
+        # upload
+        origscratch = getattr(i, "_get_system_scratchdir")
+        try:
+            setattr(i, "_get_system_scratchdir",
+                    lambda: i.scratchdir)
+            self._testInstall(g, "rhel6-kvm-stage1", "rhel6-kvm-stage2")
+        finally:
+            setattr(i, "_get_system_scratchdir", origscratch)
 
     def testFullKVMWinxp(self):
         utils.set_conn(_plainkvm)
@@ -946,5 +962,18 @@ class TestXMLConfig(unittest.TestCase):
                           virtinst.Guest.cpuset_str_to_tuple,
                           conn, "16")
 
+    def testManyVirtio(self):
+        d = VirtualDisk(conn=utils.get_conn(), bus="virtio",
+                        path="/default-pool/testvol1.img")
+
+        targetlist = []
+        for ignore in range(0, (26 * 2) + 1):
+            d.target = None
+            d.generate_target(targetlist)
+            targetlist.append(d.target)
+
+        self.assertEquals("vdaa", targetlist[26])
+        self.assertEquals("vdba", targetlist[26 * 2])
+
 if __name__ == "__main__":
     unittest.main()
