@@ -93,8 +93,8 @@ class XMLParseTest(unittest.TestCase):
 
         check("name", "TestGuest", "change_name")
         check("description", None, "Hey desc changed")
-        check("maxvcpus", 5, 30)
-        check("vcpus", 30, 22)
+        check("maxvcpus", 5, 12)
+        check("vcpus", 12, 10)
         check("cpuset", "1-3", "1-8,^6", "1-5,15")
         check("maxmemory", 400, 500)
         check("memory", 200, 1000)
@@ -102,6 +102,7 @@ class XMLParseTest(unittest.TestCase):
         check("uuid", "12345678-1234-1234-1234-123456789012",
                       "11111111-2222-3333-4444-555555555555")
         check("emulator", "/usr/lib/xen/bin/qemu-dm", "/usr/binnnn/fooemu")
+        check("hugepage", False, True)
 
         check = self._make_checker(guest.clock)
         check("offset", "utc", "localtime")
@@ -147,7 +148,7 @@ class XMLParseTest(unittest.TestCase):
         check("vendor", "Intel", "qemuvendor")
         check("threads", 2, 1)
         check("cores", 5, 3)
-        check("sockets", 4, 1)
+        check("sockets", 4, 4)
 
         check = self._make_checker(guest.cpu.features[0])
         check("name", "x2apic", "foofeat")
@@ -312,15 +313,15 @@ class XMLParseTest(unittest.TestCase):
         check("source_host", "my.source.host", "source.foo.host")
 
         check = self._make_checker(console1)
+        check("char_type", "pty")
+        check("target_type", None)
+
+        check = self._make_checker(console2)
         check("char_type", "file")
         check("source_path", "/tmp/foo.img", None)
         check("source_path", None, "/root/foo")
         check("target_type", "virtio")
 
-        check = self._make_checker(console2)
-        check("char_type", "pty")
-        check("target_type", None)
-
         check = self._make_checker(channel1)
         check("char_type", "pty")
         check("target_type", "virtio")
@@ -343,6 +344,7 @@ class XMLParseTest(unittest.TestCase):
         dev1 = guest.get_devices("controller")[0]
         dev2 = guest.get_devices("controller")[1]
         dev3 = guest.get_devices("controller")[2]
+        dev4 = guest.get_devices("controller")[3]
 
         check = self._make_checker(dev1)
         check("type", "ide")
@@ -358,6 +360,14 @@ class XMLParseTest(unittest.TestCase):
         check("type", "scsi")
         check("index", "1", "2")
 
+        check = self._make_checker(dev4)
+        check("type", "usb")
+        check("index", "3", "9")
+        check("model", "ich9-uhci3")
+
+        check = self._make_checker(dev4.get_master())
+        check("startport", "4", "2")
+
         self._alter_compare(guest.get_config_xml(), outfile)
 
     def testAlterNics(self):
@@ -567,10 +577,13 @@ class XMLParseTest(unittest.TestCase):
         dev1 = guest.get_devices(devtype)[0]
         dev2 = guest.get_devices(devtype)[1]
         dev3 = guest.get_devices(devtype)[2]
+        dev4 = guest.get_devices(devtype)[3]
 
         check = self._make_checker(dev1)
         check("type", None, "mount")
         check("mode", None, "passthrough")
+        check("driver", "handle", None)
+        check("wrpolicy", None, None)
         check("source", "/foo/bar", "/new/path")
         check("target", "/bar/baz", "/new/target")
 
@@ -583,6 +596,16 @@ class XMLParseTest(unittest.TestCase):
         check = self._make_checker(dev3)
         check("type", "mount", None)
         check("mode", "squash", None)
+        check("driver", "path", "handle")
+        check("wrpolicy", "immediate", None)
+        check("readonly", False, True)
+
+        check = self._make_checker(dev4)
+        check("type", "mount", None)
+        check("mode", "mapped", None)
+        check("driver", "path", "handle")
+        check("wrpolicy", None, "immediate")
+        check("readonly", False, True)
 
         self._alter_compare(guest.get_config_xml(), outfile)
 
@@ -663,6 +686,24 @@ class XMLParseTest(unittest.TestCase):
 
         self._alter_compare(guest.get_config_xml(), outfile)
 
+    def testAlterRedirdev(self):
+        infile  = "tests/xmlparse-xml/change-redirdev-in.xml"
+        outfile = "tests/xmlparse-xml/change-redirdev-out.xml"
+        guest = virtinst.Guest(conn=conn,
+                               parsexml=file(infile).read())
+
+        dev1 = guest.get_devices("redirdev")[0]
+        dev2 = guest.get_devices("redirdev")[1]
+
+        check = self._make_checker(dev1)
+        check("host", "foo", "bar")
+        check("service", "12", "42")
+
+        check = self._make_checker(dev2)
+        check("type", "spicevmc")
+
+        self._alter_compare(guest.get_config_xml(), outfile)
+
     def testConsoleCompat(self):
         infile  = "tests/xmlparse-xml/console-compat-in.xml"
         outfile = "tests/xmlparse-xml/console-compat-out.xml"
