@@ -69,13 +69,13 @@ def _sanitize_url(url):
 def _build_pool(conn, meter, path):
     pool = _util.lookup_pool_by_path(conn, path)
     if pool:
-        logging.debug("Existing pool '%s' found for %s" % (pool.name(), path))
+        logging.debug("Existing pool '%s' found for %s", pool.name(), path)
         pool.refresh(0)
         return pool
 
     name = _util.generate_name("boot-scratch",
                                conn.storagePoolLookupByName)
-    logging.debug("Building storage pool: path=%s name=%s" % (path, name))
+    logging.debug("Building storage pool: path=%s name=%s", path, name)
     poolbuild = Storage.DirectoryPool(conn=conn, name=name,
                                       target_path=path)
 
@@ -102,7 +102,7 @@ def _upload_file(conn, meter, destpool, src):
     name = Storage.StorageVolume.find_free_name(basename,
                                                 pool_object=destpool)
     if name != basename:
-        logging.debug("Generated non-colliding volume name %s" % name)
+        logging.debug("Generated non-colliding volume name %s", name)
 
     disk = VirtualDisk(conn=conn,
                        path=os.path.join(poolpath, name),
@@ -111,6 +111,8 @@ def _upload_file(conn, meter, destpool, src):
 
     disk.setup_dev(meter=meter)
     vol = disk.vol_object
+    if not vol:
+        raise RuntimeError(_("Failed to lookup scratch media volume"))
 
     try:
         # Register upload
@@ -205,7 +207,7 @@ class DistroInstaller(Installer.Installer):
         elif os.path.exists(os.path.abspath(val)) and is_local:
             val = os.path.abspath(val)
             logging.debug("DistroInstaller location is a local "
-                          "file/path: %s" % val)
+                          "file/path: %s", val)
 
         else:
             # Didn't determine anything about the location
@@ -229,8 +231,9 @@ class DistroInstaller(Installer.Installer):
                                 conn=self.conn,
                                 volName=stuple)
                 val = d.path
-            except Exception, e:
-                logging.debug(str(e))
+            except:
+                logging.debug("Error validating install location",
+                              exc_info=True)
                 raise ValueError(_("Checking installer location failed: "
                                    "Could not find media '%s'." % str(val)))
         elif not validated:
@@ -270,16 +273,15 @@ class DistroInstaller(Installer.Installer):
                            transient=transient)
         self.install_devices.append(disk)
 
-    def _perform_initrd_injections(self):
+    def _perform_initrd_injections(self, initrd):
         """
         Insert files into the root directory of the initial ram disk
         """
-        initrd = self._install_bootconfig.initrd
         tempdir = tempfile.mkdtemp(dir=self.scratchdir)
         os.chmod(tempdir, 0775)
 
         for filename in self._initrd_injections:
-            logging.debug("Copying %s to the initrd." % filename)
+            logging.debug("Copying %s to the initrd.", filename)
             shutil.copy(filename, tempdir)
 
         logging.debug("Appending to the initrd.")
@@ -302,6 +304,8 @@ class DistroInstaller(Installer.Installer):
     def support_remote_url_install(self):
         if not self.conn:
             return False
+        if hasattr(self.conn, "_virtinst__fake_conn"):
+            return False
         return support.check_stream_support(self.conn,
                                             support.SUPPORT_STREAM_UPLOAD)
 
@@ -310,9 +314,9 @@ class DistroInstaller(Installer.Installer):
         system_scratchdir = self._get_system_scratchdir()
 
         if (not guest.is_remote() and
-            self.scratchdir == system_scratchdir):
+            (self.is_session_uri() or self.scratchdir == system_scratchdir)):
             # We have access to system scratchdir, don't jump through hoops
-            logging.debug("Have access to local system scratchdir so"
+            logging.debug("Have access to preferred scratchdir so"
                           " nothing to upload")
             return kernel, initrd
 
@@ -367,11 +371,11 @@ class DistroInstaller(Installer.Installer):
 
         if guest.get_os_autodetect():
             if os_type:
-                logging.debug("Auto detected OS type as: %s" % os_type)
+                logging.debug("Auto detected OS type as: %s", os_type)
                 guest.os_type = os_type
 
             if (os_variant and guest.os_type == os_type):
-                logging.debug("Auto detected OS variant as: %s" % os_variant)
+                logging.debug("Auto detected OS variant as: %s", os_variant)
                 guest.os_variant = os_variant
 
         self._tmpfiles.append(kernelfn)
@@ -379,7 +383,7 @@ class DistroInstaller(Installer.Installer):
             self._tmpfiles.append(initrdfn)
 
         if self._initrd_injections:
-            self._perform_initrd_injections()
+            self._perform_initrd_injections(initrdfn)
 
         # If required, upload media to an accessible guest location
         kernelfn, initrdfn = self._upload_media(guest, meter,
