@@ -40,6 +40,7 @@ from VirtualDevice import VirtualDevice
 from VirtualDisk import VirtualDisk
 from VirtualInputDevice import VirtualInputDevice
 from VirtualCharDevice import VirtualCharDevice
+from VirtualController import VirtualControllerUSB
 from Clock import Clock
 from Seclabel import Seclabel
 from CPU import CPU
@@ -190,6 +191,7 @@ class Guest(XMLBuilderDomain.XMLBuilderDomain):
         self._uuid = None
         self._memory = None
         self._maxmemory = None
+        self._hugepage = None
         self._vcpus = 1
         self._maxvcpus = 1
         self._cpuset = None
@@ -343,6 +345,14 @@ class Guest(XMLBuilderDomain.XMLBuilderDomain):
                               xpath="./memory",
                               get_converter=lambda s, x: int(x) / 1024,
                               set_converter=lambda s, x: int(x) * 1024)
+    def get_hugepage(self):
+        return self._hugepage
+    def set_hugepage(self, val):
+        if val is None:
+            return val
+        self._hugepage = bool(val)
+    hugepage = _xml_property(get_hugepage, set_hugepage,
+                             xpath="./memoryBacking/hugepages", is_bool=True)
 
     # UUID for the guest
     def get_uuid(self):
@@ -457,8 +467,8 @@ class Guest(XMLBuilderDomain.XMLBuilderDomain):
             for ostype in self.list_os_types():
                 if (val in self._OS_TYPES[ostype]["variants"] and
                     not self._OS_TYPES[ostype]["variants"][val].get("skip")):
-                    logging.debug("Setting os type to '%s' for variant '%s'" %
-                                  (ostype, val))
+                    logging.debug("Setting os type to '%s' for variant '%s'",
+                                  ostype, val)
                     self.os_type = ostype
                     self._os_variant = val
                     found = True
@@ -772,6 +782,7 @@ class Guest(XMLBuilderDomain.XMLBuilderDomain):
             "controller": virtinst.VirtualController,
             "filesystem": virtinst.VirtualFilesystem,
             "smartcard" : virtinst.VirtualSmartCardDevice,
+            "redirdev"   : virtinst.VirtualRedirDevice,
         }
 
         # Hand off all child element parsing to relevant classes
@@ -1043,9 +1054,14 @@ class Guest(XMLBuilderDomain.XMLBuilderDomain):
         xml = add(desc_xml)
         xml = add("  <memory>%s</memory>" % (self.maxmemory * 1024))
         xml = add("  <currentMemory>%s</currentMemory>" % (self.memory * 1024))
+
         # <blkiotune>
         # <memtune>
-        # <memoryBacking>
+        if self.hugepage is True:
+            xml = add("  <memoryBacking>")
+            xml = add("    <hugepages/>")
+            xml = add("  </memoryBacking>")
+
         xml = add(self._get_vcpu_xml())
         # <cputune>
         xml = add(self.numatune.get_xml_config())
@@ -1181,12 +1197,12 @@ class Guest(XMLBuilderDomain.XMLBuilderDomain):
                                self.name)
 
         try:
-            logging.debug("Explicitly replacing guest '%s'" % self.name)
+            logging.debug("Explicitly replacing guest '%s'", self.name)
             if vm.ID() != -1:
-                logging.info("Destroying guest '%s'" % self.name)
+                logging.info("Destroying guest '%s'", self.name)
                 vm.destroy()
 
-            logging.info("Undefining guest '%s'" % self.name)
+            logging.info("Undefining guest '%s'", self.name)
             vm.undefine()
         except libvirt.libvirtError, e:
             raise RuntimeError(_("Could not remove old vm '%s': %s") %
@@ -1265,10 +1281,10 @@ class Guest(XMLBuilderDomain.XMLBuilderDomain):
         start_xml = self.get_xml_config(install=True, disk_boot=disk_boot)
         final_xml = self.get_xml_config(install=False)
 
-        logging.debug("Generated %s XML: %s" %
-                      (log_label,
-                      (start_xml and ("\n" + start_xml) or "None required")))
-        logging.debug("Generated boot XML: \n%s" % final_xml)
+        logging.debug("Generated %s XML: %s",
+                      log_label,
+                      (start_xml and ("\n" + start_xml) or "None required"))
+        logging.debug("Generated boot XML: \n%s", final_xml)
 
         return start_xml, final_xml
 
@@ -1308,7 +1324,7 @@ class Guest(XMLBuilderDomain.XMLBuilderDomain):
                 logging.debug("XML fetched from libvirt object:\n%s",
                               dom.XMLDesc(0))
             except Exception, e:
-                logging.debug("Error fetching XML from libvirt object: %s" % e)
+                logging.debug("Error fetching XML from libvirt object: %s", e)
 
         # if we connected the console, wait for it to finish
         self._waitpid_console(self._consolechild, wait)
@@ -1346,7 +1362,7 @@ class Guest(XMLBuilderDomain.XMLBuilderDomain):
         try:
             os.waitpid(console_child, 0)
         except OSError, (err_no, msg):
-            logging.debug("waitpid: %s: %s" % (err_no, msg))
+            logging.debug("waitpid: %s: %s", err_no, msg)
 
         # ensure there's time for the domain to finish destroying if the
         # install has finished or the guest crashed
@@ -1428,6 +1444,26 @@ class Guest(XMLBuilderDomain.XMLBuilderDomain):
             if hasattr(d, "virtinst_default"):
                 remove_func(d)
 
+    def add_usb_ich9_controllers(self):
+        ctrl = VirtualControllerUSB(self.conn,
+                                    model="ich9-ehci1")
+        self.add_device(ctrl)
+
+        ctrl = VirtualControllerUSB(self.conn,
+                                    model="ich9-uhci1")
+        ctrl.get_master().startport = 0
+        self.add_device(ctrl)
+
+        ctrl = VirtualControllerUSB(self.conn,
+                                    model="ich9-uhci2")
+        ctrl.get_master().startport = 2
+        self.add_device(ctrl)
+
+        ctrl = VirtualControllerUSB(self.conn,
+                                    model="ich9-uhci3")
+        ctrl.get_master().startport = 4
+        self.add_device(ctrl)
+
     def _set_defaults(self, devlist_func, remove_func, features):
         if self.installer.is_hvm():
             self._set_hvm_defaults(devlist_func, features)
