@@ -117,7 +117,7 @@ def _is_dir_searchable(uid, username, path):
         return False
 
     if proc.returncode != 0:
-        logging.debug("Cmd '%s' failed: %s" % (cmd, err))
+        logging.debug("Cmd '%s' failed: %s", cmd, err)
         return False
 
     return bool(re.search("user:%s:..x" % username, out))
@@ -164,7 +164,10 @@ def _check_if_path_managed(conn, path):
             vol = conn.storageVolLookupByPath(path)
             vol.info()
             return vol, None
-        except Exception, e:
+        except libvirt.libvirtError, e:
+            if (hasattr(libvirt, "VIR_ERR_NO_STORAGE_VOL")
+                and e.get_error_code() != libvirt.VIR_ERR_NO_STORAGE_VOL):
+                raise
             return None, e
 
     def lookup_vol_name(name):
@@ -232,9 +235,9 @@ def _build_vol_install(path, pool, size, sparse):
                            "existent volume path '%s'" % path))
 
     logging.debug("Path '%s' is target for pool '%s'. "
-                  "Creating volume '%s'." %
-                  (os.path.dirname(path), pool.name(),
-                   os.path.basename(path)))
+                  "Creating volume '%s'.",
+                  os.path.dirname(path), pool.name(),
+                  os.path.basename(path))
 
     volclass = Storage.StorageVolume.get_volume_for_pool(pool_object=pool)
     cap = (size * 1024 * 1024 * 1024)
@@ -345,7 +348,7 @@ class VirtualDisk(VirtualDevice):
             return "dir"
         return "file"
 
-    error_policies = ["none", "stop", "enospace"]
+    error_policies = ["ignore", "stop", "enospace"]
 
     @staticmethod
     def path_exists(conn, path):
@@ -386,7 +389,7 @@ class VirtualDisk(VirtualDevice):
         try:
             uid = _name_uid(username)
         except Exception, e:
-            logging.debug("Error looking up username: %s" % str(e))
+            logging.debug("Error looking up username: %s", str(e))
             return []
 
         fixlist = []
@@ -421,15 +424,22 @@ class VirtualDisk(VirtualDevice):
                                         stderr=subprocess.PIPE)
                 out, err = proc.communicate()
 
-                logging.debug("Ran command '%s'" % cmd)
+                logging.debug("Ran command '%s'", cmd)
                 if out or err:
-                    logging.debug("out=%s\nerr=%s" % (out, err))
+                    logging.debug("out=%s\nerr=%s", out, err)
 
                 if proc.returncode != 0:
                     raise ValueError(err)
             else:
+                logging.debug("Setting +x on %s", dirname)
                 mode = os.stat(dirname).st_mode
-                os.chmod(dirname, mode | stat.S_IXOTH)
+                newmode = mode | stat.S_IXOTH
+                os.chmod(dirname, newmode)
+                if os.stat(dirname).st_mode != newmode:
+                    # Trying to change perms on vfat at least doesn't work
+                    # but also doesn't seem to error. Try and detect that
+                    raise ValueError(_("Permissions on '%s' did not stick") %
+                                     dirname)
 
         fixlist = VirtualDisk.check_path_search_for_user(conn, path, username)
         if not fixlist:
@@ -449,6 +459,7 @@ class VirtualDisk(VirtualDevice):
                         raise
                     useacl = False
 
+                    logging.debug("setfacl failed, trying old fashioned way")
                     fix_perms(dirname, useacl)
             except Exception, e:
                 errdict[dirname] = str(e)
@@ -482,7 +493,7 @@ class VirtualDisk(VirtualDevice):
             template += "source/@%s='%s'])"
 
             for dtype in VirtualDisk._target_props:
-                xpath = template % (dtype, path)
+                xpath = template % (dtype, _util.xml_escape(path))
                 c += ctx.xpathEval(xpath)
 
             return c
@@ -898,7 +909,7 @@ class VirtualDisk(VirtualDevice):
         retlabel = self._selinux_label
         if not retlabel:
             retlabel = ""
-            if self.__creating_storage() and not self.__managed_storage():
+            if self.creating_storage() and not self.__managed_storage():
                 retlabel = self._expected_security_label()
             else:
                 retlabel = self._storage_security_label()
@@ -961,7 +972,10 @@ class VirtualDisk(VirtualDevice):
         elif path:
             vol_object, pool, path_is_pool = _check_if_path_managed(self.conn,
                                                                     path)
-            if pool and not vol_object and not path_is_pool:
+            if (pool and
+                not vol_object and
+                not path_is_pool and
+                not self._is_parse()):
                 vol_install = _build_vol_install(path, pool,
                                                  self.size,
                                                  self.sparse)
@@ -986,7 +1000,7 @@ class VirtualDisk(VirtualDevice):
         if not self.format:
             return
 
-        if not self.__creating_storage():
+        if not self.creating_storage():
             return
 
         if self.vol_install:
@@ -1004,7 +1018,7 @@ class VirtualDisk(VirtualDevice):
         """
         Return size of existing storage
         """
-        if self.__creating_storage():
+        if self.creating_storage():
             return
 
         if self.vol_object:
@@ -1107,7 +1121,7 @@ class VirtualDisk(VirtualDevice):
                     drvtype = _qemu_sanitize_drvtype(self.type,
                                                      self.vol_install.format)
 
-        elif self.__creating_storage():
+        elif self.creating_storage():
             if drvname == self.DRIVER_QEMU:
                 drvtype = self.DRIVER_QEMU_RAW
 
@@ -1123,23 +1137,38 @@ class VirtualDisk(VirtualDevice):
         Return bool representing if managed storage parameters have
         been explicitly specified or filled in
         """
-        return (self.vol_object != None or self.vol_install != None or
-                self._pool_object != None)
+        return bool(self.vol_object != None or
+                    self.vol_install != None or
+                    self._pool_object != None)
 
-    def __creating_storage(self):
+    def creating_storage(self):
         """
         Return True if the user requested us to create a device
         """
-        return not (self.__no_storage() or
-                    (self.__managed_storage() and
-                     self.vol_object or self._pool_object) or
-                    (self.path and os.path.exists(self.path)))
+        if self.__no_storage():
+            return False
+
+        if self.__managed_storage():
+            if self.vol_object or self._pool_object:
+                return False
+            return True
+
+        if (not self.is_remote() and
+            self.path and
+            os.path.exists(self.path)):
+            return False
+
+        return True
 
     def __no_storage(self):
         """
         Return True if no path or storage was specified
         """
-        return (not self.__managed_storage() and not self.path)
+        if self.__managed_storage():
+            return False
+        if self.path:
+            return False
+        return True
 
 
     def _storage_security_label(self):
@@ -1201,7 +1230,7 @@ class VirtualDisk(VirtualDevice):
         # - Do we need to create the storage?
 
         managed_storage = self.__managed_storage()
-        create_media = self.__creating_storage()
+        create_media = self.creating_storage()
 
         self.__set_format()
 
@@ -1348,8 +1377,8 @@ class VirtualDisk(VirtualDevice):
             clone_block_size = 1024 * 1024 * 10
             sparse = False
 
-        logging.debug("Local Cloning %s to %s, sparse=%s, block_size=%s" %
-                      (self.clone_path, self.path, sparse, clone_block_size))
+        logging.debug("Local Cloning %s to %s, sparse=%s, block_size=%s",
+                      self.clone_path, self.path, sparse, clone_block_size)
 
         zeros = '\0' * 4096
 
@@ -1406,7 +1435,7 @@ class VirtualDisk(VirtualDevice):
         if not progresscb:
             progresscb = progress.BaseMeter()
 
-        if self.__creating_storage() or self.clone_path:
+        if self.creating_storage() or self.clone_path:
             self._do_create_storage(progresscb)
 
         # Relabel storage if it was requested
@@ -1417,8 +1446,8 @@ class VirtualDisk(VirtualDevice):
             elif not self._security_can_fix():
                 logging.debug("Can't fix selinux context in this case.")
             else:
-                logging.debug("Changing path=%s selinux label %s -> %s" %
-                              (self.path, storage_label, self.selinux_label))
+                logging.debug("Changing path=%s selinux label %s -> %s",
+                              self.path, storage_label, self.selinux_label)
                 _util.selinux_setfilecon(self.path, self.selinux_label)
 
     def _get_xml_config(self, disknode=None):
@@ -1522,7 +1551,7 @@ class VirtualDisk(VirtualDevice):
         if self.vol_install:
             return self.vol_install.is_size_conflict()
 
-        if not self.__creating_storage():
+        if not self.creating_storage():
             return (False, None)
 
         ret = False
@@ -1657,17 +1686,17 @@ class VirtualDisk(VirtualDevice):
 
         return can_fix
 
-    def _get_target_type(self):
+    def get_target_prefix(self):
         """
-        Returns the suggested disk target prefix (hd, xvd, sd ...) from
-        the passed parameters.
+        Returns the suggested disk target prefix (hd, xvd, sd ...) for the
+        disk.
         @returns: str prefix, or None if no reasonable guess can be made
         """
         # The upper limits here aren't necessarilly 1024, but let the HV
         # error as appropriate.
         if self.bus == "virtio":
             return ("vd", 1024)
-        elif self.bus == "scsi" or self.bus == "usb":
+        elif self.bus in ["sata", "scsi", "usb"]:
             return ("sd", 1024)
         elif self.bus == "xen":
             return ("xvd", 1024)
@@ -1693,7 +1722,7 @@ class VirtualDisk(VirtualDevice):
         # Only use these targets if there are no other options
         except_targets = ["hdc"]
 
-        prefix, maxnode = self._get_target_type()
+        prefix, maxnode = self.get_target_prefix()
         if prefix is None:
             raise ValueError(_("Cannot determine device bus/type."))
 
@@ -1703,9 +1732,32 @@ class VirtualDisk(VirtualDevice):
                 self.target = "hdc"
                 return self.target
 
+        if maxnode > (26 * 26 * 26):
+            raise RuntimeError("maxnode value is too high")
+
         # Regular scanning
-        for i in range(maxnode):
-            gen_t = "%s%c" % (prefix, ord('a') + i)
+        for i in range(1, maxnode + 1):
+            gen_t = prefix
+
+            tmp = i
+            digits = []
+            for factor in range(0, 3):
+                amt = (tmp % (26 ** (factor + 1))) / (26 ** factor)
+                if amt == 0 and tmp >= (26 ** (factor + 1)):
+                    amt = 26
+                tmp -= amt
+                digits.insert(0, amt)
+
+            seen_valid = False
+            for digit in digits:
+                if digit == 0:
+                    if not seen_valid:
+                        continue
+                    digit = 1
+
+                seen_valid = True
+                gen_t += "%c" % (ord('a') + digit - 1)
+
             if gen_t in except_targets:
                 continue
             if gen_t not in skip_targets:
