@@ -28,7 +28,6 @@ import os
 import re
 import commands
 import logging
-import traceback
 import platform
 import subprocess
 
@@ -152,7 +151,7 @@ def validate_uuid(val):
 
 def validate_name(name_type, val, lencheck=False):
     if type(val) is not str or len(val) == 0:
-        raise ValueError(_("%s name must be a string"))
+        raise ValueError(_("%s name must be a string") % name_type)
 
     if lencheck:
         if len(val) > 50:
@@ -201,7 +200,7 @@ def fetch_all_guests(conn):
             active.append(vm)
         except libvirt.libvirtError:
             # guest probably in process of dieing
-            logging.warn("Failed to lookup active domain id %d" % i)
+            logging.warn("Failed to lookup active domain id %d", i)
 
     # Get all inactive VMs
     names = conn.listDefinedDomains()
@@ -211,27 +210,10 @@ def fetch_all_guests(conn):
             inactive.append(vm)
         except:
             # guest probably in process of dieing
-            logging.warn("Failed to lookup inactive domain %d" % name)
+            logging.warn("Failed to lookup inactive domain %d", name)
 
     return (active, inactive)
 
-def log_exception(msg=""):
-    """
-    Log the most recent backtrace at the DEBUG level, rather than the
-    ERROR level (which goes to stderr when used with the cli tools)
-
-    @param msg: Option message to log before the backtrace
-    """
-    msg = str(msg)
-    tb = "".join(traceback.format_exc()).strip()
-    if tb != "None":
-        if msg:
-            msg += "\n"
-        msg += tb
-
-    if msg:
-        logging.debug(msg)
-
 def set_xml_path(xml, path, newval):
     """
     Set the passed xml xpath to the new value
@@ -317,8 +299,8 @@ def selinux_restorecon(path):
         try:
             selinux.restorecon(path)
         except Exception, e:
-            logging.debug("Restoring context for '%s' failed: %s" % (path,
-                                                                     str(e)))
+            logging.debug("Restoring context for '%s' failed: %s",
+                          path, str(e))
 def selinux_getfilecon(path):
     if have_selinux():
         return selinux.getfilecon(path)[1]
@@ -500,7 +482,7 @@ def find_xkblayout(path):
     try:
         f = open(path, "r")
     except IOError, e:
-        logging.debug('Could not open "%s": %s ' % (path, str(e)))
+        logging.debug('Could not open "%s": %s ', path, str(e))
     else:
         keymap_re = re.compile(r'\s*XKBLAYOUT="(?P<kt>[a-z-]+)"')
         for line in f:
@@ -509,7 +491,7 @@ def find_xkblayout(path):
                 kt = m.group('kt')
                 break
         else:
-            logging.debug("Didn't find keymap in '%s'!" % path)
+            logging.debug("Didn't find keymap in '%s'!", path)
         f.close()
     return kt
 
