@@ -35,7 +35,6 @@ import libvirt
 
 import virtinst
 from virtinst import _util
-from _util import log_exception
 from _util import listify
 from virtinst import _gettext as _
 
@@ -171,7 +170,6 @@ def setupLogging(appname, debug=False, do_quiet=False):
     dateFormat = "%a, %d %b %Y %H:%M:%S"
     fileFormat = ("[%(asctime)s " + appname + " %(process)d] "
                   "%(levelname)s (%(module)s:%(lineno)d) %(message)s")
-    streamDebugFormat = "%(asctime)s %(levelname)-8s %(message)s"
     streamErrorFormat = "%(levelname)-8s %(message)s"
     filename = os.path.join(vi_dir, appname + ".log")
 
@@ -192,7 +190,7 @@ def setupLogging(appname, debug=False, do_quiet=False):
     streamHandler = VirtStreamHandler(sys.stderr)
     if debug:
         streamHandler.setLevel(logging.DEBUG)
-        streamHandler.setFormatter(logging.Formatter(streamDebugFormat,
+        streamHandler.setFormatter(logging.Formatter(fileFormat,
                                                      dateFormat))
     else:
         if quiet:
@@ -207,7 +205,7 @@ def setupLogging(appname, debug=False, do_quiet=False):
     def libvirt_callback(ignore, err):
         if err[3] != libvirt.VIR_ERR_ERROR:
             # Don't log libvirt errors: global error handler will do that
-            logging.warn("Non-error from libvirt: '%s'" % err[2])
+            logging.warn("Non-error from libvirt: '%s'", err[2])
     libvirt.registerErrorHandler(f=libvirt_callback, ctx=None)
 
     # Register python error handler to log exceptions
@@ -219,7 +217,7 @@ def setupLogging(appname, debug=False, do_quiet=False):
     sys.excepthook = exception_log
 
     # Log the app command string
-    logging.debug("Launched with command line:\n%s" % " ".join(sys.argv))
+    logging.debug("Launched with command line:\n%s", " ".join(sys.argv))
 
 
 #######################################
@@ -250,7 +248,7 @@ def _open_test_uri(uri):
         xml = re.sub("machine type='.*'", "", xml)
         xml = re.sub(">exe<", ">hvm<", xml)
 
-        logging.debug("virtinst test sanitizing diff\n:%s" %
+        logging.debug("virtinst test sanitizing diff\n:%s",
                       "\n".join(difflib.unified_diff(orig.split("\n"),
                                                      xml.split("\n"))))
         return xml
@@ -313,6 +311,8 @@ def _open_test_uri(uri):
             return ver
         libvirt.getVersion = newlibversion
 
+    setattr(conn, "_virtinst__fake_conn", True)
+
     return conn
 
 def getConnection(uri):
@@ -323,9 +323,9 @@ def getConnection(uri):
     if _is_virtinst_test_uri(uri):
         return _open_test_uri(uri)
 
-    logging.debug("Requesting libvirt URI %s" % (uri or "default"))
+    logging.debug("Requesting libvirt URI %s", (uri or "default"))
     conn = open_connection(uri)
-    logging.debug("Received libvirt URI %s" % conn.getURI())
+    logging.debug("Received libvirt URI %s", conn.getURI())
 
     return conn
 
@@ -345,7 +345,7 @@ def do_creds(creds, cbdata):
     try:
         return _do_creds(creds, cbdata)
     except:
-        log_exception("Error in creds callback.")
+        logging.debug("Error in creds callback.", exc_info=True)
         raise
 
 def _do_creds(creds, cbdata_ignore):
@@ -366,7 +366,7 @@ def _do_creds_polkit(action):
     if os.getuid() == 0:
         logging.debug("Skipping policykit check as root")
         return 0 # Success
-    logging.debug("Doing policykit for %s" % action)
+    logging.debug("Doing policykit for %s", action)
 
     import subprocess
     import commands
@@ -374,13 +374,13 @@ def _do_creds_polkit(action):
     bin_path = "/usr/bin/polkit-auth"
 
     if not os.path.exists(bin_path):
-        logging.debug("%s not present, skipping polkit auth." % bin_path)
+        logging.debug("%s not present, skipping polkit auth.", bin_path)
         return 0
 
     cmdstr = "%s %s" % (bin_path, "--explicit")
     output = commands.getstatusoutput(cmdstr)
     if output[1].count(action):
-        logging.debug("User already authorized for %s." % action)
+        logging.debug("User already authorized for %s.", action)
         # Hide spurious output from polkit-auth
         popen_stdout = subprocess.PIPE
         popen_stderr = subprocess.PIPE
@@ -399,9 +399,9 @@ def _do_creds_polkit(action):
     out, err = proc.communicate()
 
     if out and popen_stdout:
-        logging.debug("polkit-auth stdout: %s" % out)
+        logging.debug("polkit-auth stdout: %s", out)
     if err and popen_stderr:
-        logging.debug("polkit-auth stderr: %s" % err)
+        logging.debug("polkit-auth stderr: %s", err)
 
     return 0
 
@@ -420,8 +420,7 @@ def _do_creds_authname(creds):
             import getpass
             res = getpass.getpass(prompt)
         else:
-            logging.debug("Unknown auth type in creds callback: %d" %
-                          credtype)
+            logging.debug("Unknown auth type in creds callback: %d", credtype)
             return -1
 
         cred[retindex] = res
@@ -438,7 +437,7 @@ def fail(msg, do_exit=True):
     Convenience function when failing in cli app
     """
     logging.error(msg)
-    log_exception()
+    logging.debug("", exc_info=True)
     if do_exit:
         _fail_exit()
 
@@ -485,7 +484,7 @@ def build_default_pool(guest):
         return
 
     try:
-        logging.debug("Attempting to build default pool with target '%s'" %
+        logging.debug("Attempting to build default pool with target '%s'",
                       DEFAULT_POOL_PATH)
         defpool = virtinst.Storage.DirectoryPool(conn=guest.conn,
                                                  name=DEFAULT_POOL_NAME,
@@ -524,6 +523,9 @@ def is_prompt():
     return doprompt
 
 def yes_or_no_convert(s):
+    if s is None:
+        return None
+
     s = s.lower()
     if s in ("y", "yes", "1", "true", "t"):
         return True
@@ -549,6 +551,7 @@ def prompt_for_input(noprompt_err, prompt="", val=None, failed=False):
         fail(noprompt_err)
 
     print_stdout(prompt + " ", do_force=True)
+    sys.stdout.flush()
     return sys.stdin.readline().strip()
 
 def prompt_for_yes_or_no(warning, question):
@@ -591,7 +594,7 @@ def prompt_loop(prompt_txt, noprompt_err, passed_val, obj, param_name,
             setattr(obj, param_name, passed_val)
             break
         except (ValueError, RuntimeError), e:
-            logging.error(err_txt % e)
+            logging.error(err_txt, e)
             passed_val = None
             failed = True
 
@@ -827,10 +830,10 @@ def get_cpuset(guest, cpuset):
         try:
             tmpset = Guest.generate_cpuset(conn, guest.memory)
         except Exception, e:
-            logging.debug("Not setting cpuset", str(e))
+            logging.debug("Not setting cpuset: %s", str(e))
 
         if tmpset:
-            logging.debug("Auto cpuset is: %s" % tmpset)
+            logging.debug("Auto cpuset is: %s", tmpset)
             guest.cpuset = tmpset
 
     return
@@ -944,7 +947,7 @@ def digest_graphics(guest, options, default_override=None):
     if keymap:
         optstr += ",keymap=%s" % keymap
 
-    logging.debug("--graphics compat generated: %s" % optstr)
+    logging.debug("--graphics compat generated: %s", optstr)
     return [optstr]
 
 def get_graphics(guest, graphics):
@@ -994,6 +997,26 @@ def get_smartcard(guest, sc_opts):
         if dev:
             guest.add_device(dev)
 
+def get_controller(guest, sc_opts):
+    for sc in listify(sc_opts):
+        try:
+            dev = parse_controller(guest, sc)
+        except Exception, e:
+            fail(_("Error in controller device parameters: %s") % str(e))
+
+        if dev:
+            guest.add_device(dev)
+
+def get_redirdev(guest, sc_opts):
+    for sc in listify(sc_opts):
+        try:
+            dev = parse_redirdev(guest, sc)
+        except Exception, e:
+            fail(_("Error in redirdev device parameters: %s") % str(e))
+
+        if dev:
+            guest.add_device(dev)
+
 #############################
 # Common CLI option/group   #
 #############################
@@ -1062,6 +1085,9 @@ def add_net_option(devg):
              "--network network=mynet,model=virtio,mac=00:11..."))
 
 def add_device_options(devg):
+    devg.add_option("", "--controller", dest="controller", action="append",
+                    help=_("Configure a guest controller device. Ex:\n"
+                           "--controller type=usb,model=ich9-ehci1"))
     devg.add_option("", "--serial", dest="serials", action="append",
                     help=_("Configure a guest serial device"))
     devg.add_option("", "--parallel", dest="parallels", action="append",
@@ -1083,6 +1109,9 @@ def add_device_options(devg):
     devg.add_option("", "--smartcard", dest="smartcard", action="append",
                     help=_("Configure a guest smartcard device. Ex:\n"
                            "--smartcard mode=passthrough"))
+    devg.add_option("", "--redirdev", dest="redirdev", action="append",
+                    help=_("Configure a guest redirection device. Ex:\n"
+                           "--redirdev usb,type=tcp,server=192.168.1.1:4000"))
 
 def add_gfx_option(devg):
     devg.add_option("", "--graphics", dest="graphics", action="append",
@@ -1146,9 +1175,9 @@ def parse_optstr_tuples(optstr, compress_first=False):
         opt_val = None
         if opt.count("="):
             opt_type, opt_val = opt.split("=", 1)
-            optlist.append((opt_type.lower(), opt_val))
+            optlist.append((opt_type, opt_val))
         else:
-            optlist.append((opt.lower(), None))
+            optlist.append((opt, None))
 
     return optlist
 
@@ -1382,6 +1411,7 @@ def parse_security(guest, security):
     # Beware, adding boolean options here could upset label comma handling
     mode = get_opt_param(opts, "type")
     label = get_opt_param(opts, "label")
+    relabel = yes_or_no_convert(get_opt_param(opts, "relabel"))
 
     # Try to fix up label if it contained commas
     if label:
@@ -1408,6 +1438,9 @@ def parse_security(guest, security):
     if mode:
         secmodel.type = mode
 
+    if relabel:
+        secmodel.relabel = relabel
+
     if opts:
         raise ValueError(_("Unknown options %s") % opts.keys())
 
@@ -1468,8 +1501,8 @@ def _parse_disk_source(guest, path, pool, vol, size, fmt, sparse):
                                "vol=poolname/volname"))
         vollist = vol.split("/")
         voltuple = (vollist[0], vollist[1])
-        logging.debug("Parsed volume: as pool='%s' vol='%s'" %
-                      (voltuple[0], voltuple[1]))
+        logging.debug("Parsed volume: as pool='%s' vol='%s'",
+                      voltuple[0], voltuple[1])
         if voltuple[0] == DEFAULT_POOL_NAME:
             build_default_pool(guest)
 
@@ -1679,6 +1712,36 @@ def parse_graphics(guest, optstring, dev=None):
 
     return dev
 
+#######################
+# --controller parsing #
+#######################
+
+def parse_controller(guest, optstring, dev=None):
+    if optstring is None:
+        return None
+
+    # Peel the mode off the front
+    opts = parse_optstr(optstring, remove_first="type")
+    ctrltype = get_opt_param(opts, "type")
+    address = get_opt_param(opts, "address")
+    master = get_opt_param(opts, "master")
+
+    if not dev:
+        cl = virtinst.VirtualController.get_class_for_type(ctrltype)
+        dev = cl(guest.conn, model=opts.get("model"))
+
+    set_param = _build_set_param(dev, opts)
+
+    set_param("model", "model")
+    set_param("index", "index")
+    dev.set_address(address)
+    if master:
+        dev.set_master(master)
+    if opts:
+        raise ValueError(_("Unknown options %s") % opts.keys())
+
+    return dev
+
 #######################
 # --smartcard parsing #
 #######################
@@ -1705,6 +1768,42 @@ def parse_smartcard(guest, optstring, dev=None):
 
     return dev
 
+######################
+# --redirdev parsing #
+######################
+
+def parse_redirdev(guest, optstring, dev=None):
+    if optstring is None:
+        return None
+
+    # Peel the mode off the front
+    opts = parse_optstr(optstring, remove_first="bus")
+    bus = get_opt_param(opts, "bus")
+    stype = get_opt_param(opts, "type")
+    server = get_opt_param(opts, "server")
+
+    if bus == "none":
+        return None
+
+    if not dev:
+        dev = virtinst.VirtualRedirDevice(bus=bus,
+                                          stype=stype,
+                                          conn=guest.conn)
+
+    if stype == "spicevmc" and server:
+        raise ValueError(_("The server option is invalid with spicevmc redirection"))
+
+    if stype == "tcp" and not server:
+        raise ValueError(_("The server option is missing for TCP redirection"))
+
+    if server:
+        dev.parse_friendly_server(server)
+
+    if opts:
+        raise ValueError(_("Unknown options %s") % opts.keys())
+
+    return dev
+
 ######################
 # --watchdog parsing #
 ######################
