@@ -364,6 +364,25 @@ OS_TYPES = {
         }
     },
 
+    "opensuse11": {
+        "label": "openSuse 11",
+        "distro": "suse",
+        "supported": True,
+        "devices" : {
+            DISK : VIRTIO_DISK,
+            NET  : VIRTIO_NET,
+        },
+    },
+    "opensuse12": {
+        "label": "openSuse 12",
+        "distro": "suse",
+        "supported": True,
+        "devices" : {
+            DISK : VIRTIO_DISK,
+            NET  : VIRTIO_NET,
+        },
+    },
+
     "sles10": {
         "label": "Suse Linux Enterprise Server",
         "distro": "suse",
@@ -407,6 +426,18 @@ OS_TYPES = {
         },
     },
 
+    "mageia1": {
+        "label": "Mageia 1 and later",
+        "distro": "mageia",
+        "supported": True,
+        "devices" : {
+            DISK : VIRTIO_DISK,
+            NET  : VIRTIO_NET,
+            INPUT: USB_TABLET,
+        },
+    },
+
+
     "debianetch": {
         "label": "Debian Etch",
         "distro": "debian",
@@ -433,6 +464,17 @@ OS_TYPES = {
             INPUT: USB_TABLET,
         }
     },
+    "debianwheezy": {
+        "label": "Debian Wheezy",
+        "distro": "debian",
+        "sortby": "debian7",
+        "supported": True,
+        "devices" : {
+                   DISK : VIRTIO_DISK,
+                   NET  : VIRTIO_NET,
+                   INPUT: USB_TABLET,
+        }
+    },
 
     "ubuntuhardy": {
         "label": "Ubuntu 8.04 LTS (Hardy Heron)",
